#ifndef __CONFIG_H__
#define __CONFIG_H__

#define VERSION        "6.5.48"

/* Set the QML module URI from CMake */
/* #undef PIN_SCREENSHOTS_QML_URI */
/* Set the QML module URI from CMake */
static const auto PIN_SCREENSHOTS_QML_PATH = u"";

/* Set the QML module URI from CMake */
#define SCREENRECORDERCORE_QML_URI "com.deepin.voicenote.private"
/* Set the QML module URI from CMake */
static const auto SCREENRECORDERCORE_QML_PATH = u"qrc:/com/deepin/voicenote/private";

#endif  /* __CONFIG_H__ */
