// Copyright (C) 2015 ~ 2018 Deepin Technology Co., Ltd.
// SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: GPL-3.0-or-later

/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Command line was: qdbusxml2cpp -c DBusLogin1Manager -p dbuslogin1manager orgfreedesktop.login1.xml
 *
 * qdbusxml2cpp is Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#include "dbuslogin1manager.h"
#include <QDebug>

/*
 * Implementation of interface class DBusLogin1Manager
 */

DBusLogin1Manager::DBusLogin1Manager(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent)
    : QDBusAbstractInterface(service, path, staticInterfaceName(), connection, parent)
{
    // qInfo() << "DBusLogin1Manager constructor called with service:" << service << "path:" << path;
    QDBusConnection::systemBus().connect(this->service(), this->path(), "org.freedesktop.DBus.Properties", "PropertiesChanged", "sa{sv}as", this, SLOT(__propertyChanged__(QDBusMessage)));
    Inhibit::registerMetaType();
    UserInfo::registerMetaType();
    SeatInfo::registerMetaType();
    SessionInfo::registerMetaType();
    // qInfo() << "DBusLogin1Manager constructor finished";
}

DBusLogin1Manager::~DBusLogin1Manager()
{
    // qInfo() << "DBusLogin1Manager destructor called";
    QDBusConnection::systemBus().disconnect(this->service(), this->path(), "org.freedesktop.DBus.Properties", "PropertiesChanged", "sa{sv}as", this, SLOT(__propertyChanged__(QDBusMessage)));
    // qInfo() << "DBusLogin1Manager destructor finished";
}
